import logging
#
# from .mixin.caption import CaptionMixin
# from .mixin.channel import ChannelMixin
# from .mixin.comment import CommentThreadsMixin
# from .mixin.search import SearchMixin
#
# __VERSION__ = "0.0.1"
#
# from ytvs.mixin.search import SearchMixin
# from .mixin.video import VideoMixin
#
# DEFAULT_LOGGER = logging.getLogger("youtubeapi")
#
#
# class Client(
#     SearchMixin,
#     VideoMixin,
#     ChannelMixin,
#     CommentThreadsMixin,
#     CaptionMixin
# ):
#     pass



"""
    These are playlist related models.
"""
from dataclasses import dataclass, field
from typing import Optional, List

from .base import BaseModel
from .common import BaseApiResponse, BaseResource, Localized, Player, Thumbnails
from .mixins import DatetimeTimeMixin


@dataclass
class PlaylistContentDetails(BaseModel):
    """
    A class representing playlist's content details info.

    Refer: https://developers.google.com/youtube/v3/docs/playlists#contentDetails
    """

    itemCount: Optional[int] = field(default=None)


@dataclass
class PlaylistSnippet(BaseModel, DatetimeTimeMixin):
    """
    A class representing the playlist snippet info.

    Refer: https://developers.google.com/youtube/v3/docs/playlists#snippet
    """

    publishedAt: Optional[str] = field(default=None, repr=False)
    channelId: Optional[str] = field(default=None, repr=False)
    title: Optional[str] = field(default=None)
    description: Optional[str] = field(default=None)
    thumbnails: Optional[Thumbnails] = field(default=None, repr=False)
    channelTitle: Optional[str] = field(default=None, repr=False)
    defaultLanguage: Optional[str] = field(default=None, repr=False)
    localized: Optional[Localized] = field(default=None, repr=False)


@dataclass
class PlaylistStatus(BaseModel):
    """
    A class representing the playlist status info.

    Refer: https://developers.google.com/youtube/v3/docs/playlists#status
    """

    privacyStatus: Optional[str] = field(default=None)


@dataclass
class Playlist(BaseResource):
    """
    A class representing the playlist info.

    Refer: https://developers.google.com/youtube/v3/docs/playlists
    """

    snippet: Optional[PlaylistSnippet] = field(default=None, repr=False)
    status: Optional[PlaylistStatus] = field(default=None, repr=False)
    contentDetails: Optional[PlaylistContentDetails] = field(default=None, repr=False)
    player: Optional[Player] = field(default=None, repr=False)


@dataclass
class PlaylistListResponse(BaseApiResponse):
    """
    A class representing the playlist's retrieve response info.

    Refer: https://developers.google.com/youtube/v3/docs/playlists/list#response_1
    """

    items: Optional[List[Playlist]] = field(default=None, repr=False)



"""
    These are activity related models.
"""

from dataclasses import dataclass, field
from typing import List, Optional

from .base import BaseModel
from .common import BaseApiResponse, BaseResource, ResourceId, Thumbnails
from .mixins import DatetimeTimeMixin


@dataclass
class ActivityContentDetailsUpload(BaseModel):
    """
    A class representing the activity contentDetails upload resource info.

    Refer: https://developers.google.com/youtube/v3/docs/activities#contentDetails.upload
    """

    videoId: Optional[str] = field(default=None)


@dataclass
class ActivityContentDetailsLike(BaseModel):
    """
    A class representing the activity contentDetails like resource info.

    Refer: https://developers.google.com/youtube/v3/docs/activities#contentDetails.like
    """

    resourceId: Optional[ResourceId] = field(default=None)


@dataclass
class ActivityContentDetailsFavorite(BaseModel):
    """
    A class representing the activity contentDetails favorite resource info.

    Refer: https://developers.google.com/youtube/v3/docs/activities#contentDetails.favorite
    """

    resourceId: Optional[ResourceId] = field(default=None)


@dataclass
class ActivityContentDetailsComment(BaseModel):
    """
    A class representing the activity contentDetails comment resource info.

    Refer: https://developers.google.com/youtube/v3/docs/activities#contentDetails.comment
    """

    resourceId: Optional[ResourceId] = field(default=None)


@dataclass
class ActivityContentDetailsSubscription(BaseModel):
    """
    A class representing the activity contentDetails subscription resource info.

    Refer: https://developers.google.com/youtube/v3/docs/activities#contentDetails.subscription
    """

    resourceId: Optional[ResourceId] = field(default=None)


@dataclass
class ActivityContentDetailsPlaylistItem(BaseModel):
    """
    A class representing the activity contentDetails playlistItem resource info.

    Refer: https://developers.google.com/youtube/v3/docs/activities#contentDetails.playlistItem
    """

    resourceId: Optional[ResourceId] = field(default=None)
    playlistId: Optional[str] = field(default=None)
    playlistItemId: Optional[str] = field(default=None)


@dataclass
class ActivityContentDetailsRecommendation(BaseModel):
    """
    A class representing the activity contentDetails recommendation resource info.

    Refer: https://developers.google.com/youtube/v3/docs/activities#contentDetails.recommendation
    """

    resourceId: Optional[ResourceId] = field(default=None)
    reason: Optional[str] = field(default=None)
    seedResourceId: Optional[ResourceId] = field(default=None)


@dataclass
class ActivityContentDetailsBulletin(BaseModel):
    """
    A class representing the activity contentDetails bulletin resource info.

    Refer: https://developers.google.com/youtube/v3/docs/activities#contentDetails.bulletin
    """

    resourceId: Optional[ResourceId] = field(default=None)


@dataclass
class ActivityContentDetailsSocial(BaseModel):
    """
    A class representing the activity contentDetails social resource info.

    Refer: https://developers.google.com/youtube/v3/docs/activities#contentDetails.social
    """

    type: Optional[str] = field(default=None)
    resourceId: Optional[ResourceId] = field(default=None)
    author: Optional[str] = field(default=None)
    referenceUrl: Optional[str] = field(default=None)
    imageUrl: Optional[str] = field(default=None)


@dataclass
class ActivityContentDetailsChannelItem(BaseModel):
    """
    A class representing the activity contentDetails channelItem resource info.

    Refer: https://developers.google.com/youtube/v3/docs/activities#contentDetails.channelItem
    """

    resourceId: Optional[ResourceId] = field(default=None)


@dataclass
class ActivitySnippet(BaseModel, DatetimeTimeMixin):
    """
    A class representing the activity snippet resource info.

    Refer: https://developers.google.com/youtube/v3/docs/activities#snippet
    """

    publishedAt: Optional[str] = field(default=None, repr=False)
    channelId: Optional[str] = field(default=None, repr=False)
    title: Optional[str] = field(default=None)
    description: Optional[str] = field(default=None)
    thumbnails: Optional[Thumbnails] = field(default=None, repr=False)
    channelTitle: Optional[str] = field(default=None, repr=False)
    type: Optional[str] = field(default=None, repr=False)
    groupId: Optional[str] = field(default=None, repr=False)


@dataclass
class ActivityContentDetails(BaseModel):
    """
    A class representing the activity contentDetails resource info.

    Refer: https://developers.google.com/youtube/v3/docs/activities#contentDetails
    """

    upload: Optional[ActivityContentDetailsUpload] = field(default=None)
    like: Optional[ActivityContentDetailsLike] = field(default=None, repr=False)
    favorite: Optional[ActivityContentDetailsFavorite] = field(default=None, repr=False)
    comment: Optional[ActivityContentDetailsComment] = field(default=None, repr=False)
    subscription: Optional[ActivityContentDetailsSubscription] = field(
        default=None, repr=False
    )
    playlistItem: Optional[ActivityContentDetailsPlaylistItem] = field(
        default=None, repr=False
    )
    recommendation: Optional[ActivityContentDetailsRecommendation] = field(
        default=None, repr=False
    )
    bulletin: Optional[ActivityContentDetailsBulletin] = field(default=None, repr=False)
    social: Optional[ActivityContentDetailsSocial] = field(default=None, repr=False)
    channelItem: Optional[ActivityContentDetailsChannelItem] = field(
        default=None, repr=False
    )


@dataclass
class Activity(BaseResource):
    """
    A class representing the activity resource info.

    Refer: https://developers.google.com/youtube/v3/docs/activities
    """

    snippet: Optional[ActivitySnippet] = field(default=None)
    contentDetails: Optional[ActivityContentDetails] = field(default=None, repr=False)


@dataclass
class ActivityListResponse(BaseApiResponse):
    """
    A class representing the activity response info.

    Refer: https://developers.google.com/youtube/v3/docs/activities/list#response_1
    """

    items: Optional[List[Activity]] = field(default=None, repr=False)



"""
    Base resource class.
"""
from typing import Optional, TYPE_CHECKING

if TYPE_CHECKING:
    from pyyoutube import Client  # pragma: no cover


class Resource:
    """Resource base class"""

    def __init__(self, client: Optional["Client"] = None):
        self._client = client

    @property
    def access_token(self):
        return self._client.access_token

    @property
    def api_key(self):
        return self._client.api_key



from enum import Enum


class Order(Enum):
    """
    YouTube 검색 결과의 정렬 순서를 정의하는 Enum 클래스.
    """
    DATE = 'date'
    RATING = 'rating'
    RELEVANCE = 'relevance'
    VIEW_COUNT = 'viewCount'
    VIDEO_COUNT = 'videoCount'


class SearchType(Enum):
    """
    YouTube 검색 유형을 정의하는 Enum 클래스.
    """
    # VIDEO = 'SBBA'
    VIDEO = 'video'


class VideoDuration(Enum):
    """
    YouTube 검색에서 비디오 지속 시간을 필터링하기 위한 Enum 클래스.
    """
    ANY = 'any'
    SHORT = 'short'
    MEDIUM = 'medium'
    LONG = 'long'

    # ANY = ''
    # SHORT = 'BGAE='
    # MEDIUM = 'BGAM='
    # LONG = 'BGAI='


# class Order(Enum):
#     DATE = 'CAI'
#     RATING = 'CAE'
#     RELEVANCE = 'CAA'
#     VIEW_COUNT = 'CAM'
#     VIDEO_COUNT = 'CAM'
#
#
# class SearchType(Enum):
#     VIDEO = 'SBBA'
#
#
# class VideoDuration(Enum):
#     ANY = ''
#     SHORT = 'BGAE='
#     MEDIUM = 'BGAM='
#     LONG = 'BGAI='
#

def transform_order_to_code(order: Order):
    """
    주어진 Order Enum을 YouTube API 요청 코드로 변환.
    :param order: 변환할 Order Enum입니다.
    :return: YouTube API 요청에 사용될 문자열 코드.
    """
    if order == Order.DATE.value:
        return 'CAI'
    elif order == Order.RATING.value:
        return 'CAE'
    elif order == Order.RELEVANCE.value:
        return 'CAA'
    elif order == Order.VIEW_COUNT.value:
        return 'CAM'
    raise ValueError(f"not supported order: {order}")


def transform_search_type_to_code(order: Order, is_any_video_duration: bool):
    """
    주어진 SearchType Enum을 YouTube API 요청 코드로 변환.
    :param order: 변환할 SearchType Enum.
    :return: YouTube API 요청에 사용될 문자열 코드.
    """

    if order == SearchType.VIDEO.value:
        if is_any_video_duration:
            return 'SAhAB'
        else:
            return 'SBBA'
    return ''


# def transform_search_type_to_code(order: Order):
#     """
#     주어진 SearchType Enum을 YouTube API 요청 코드로 변환.
#     :param order: 변환할 SearchType Enum.
#     :return: YouTube API 요청에 사용될 문자열 코드.
#     """
#     if order == SearchType.VIDEO.value:
#         return 'CAI'
#     return


def transform_video_duration_to_code(duration: VideoDuration):
    """
    주어진 VideoDuration Enum을 YouTube API 요청 코드로 변환.
    :param duration: 변환할 VideoDuration Enum.
    :return: YouTube API 요청에 사용될 문자열 코드.
    """
    if duration == VideoDuration.ANY.value:
        return ''
    elif duration == VideoDuration.SHORT.value:
        return 'BGAE='
    elif duration == VideoDuration.MEDIUM.value:
        return 'BGAM='
    elif duration == VideoDuration.LONG.value:
        return 'BGAI='
    raise ValueError(f"not supported video duration: {duration}")


class FilterBuilder:
    """
        YouTube 검색 필터를 구성하기 위한 클래스입니다.
    """

    def __init__(self):
        self._order = Order.RELEVANCE.value
        self._search_type = SearchType.VIDEO.value
        self._video_duration = VideoDuration.ANY.value

    def setOrder(self, order: Order):
        self._order = order
        return self

    def setSearchType(self, search_type: SearchType):
        self._search_type = search_type

    def setVideoDuration(self, video_duration: VideoDuration):
        self._video_duration = video_duration

    def build(self):
        order_code = transform_order_to_code(self._order)
        is_any_video_duration = self._video_duration == 'any'
        search_type_code = transform_search_type_to_code(self._search_type, is_any_video_duration)
        video_duratin_code = transform_video_duration_to_code(self._video_duration)

        return f'{order_code}{search_type_code}{video_duratin_code}&hl=ko'



from .activities import ActivitiesResource  # noqa
from .captions import CaptionsResource  # noqa
from .channel_banners import ChannelBannersResource  # noqa
from .channels import ChannelsResource  # noqa
from .channel_sections import ChannelSectionsResource  # noqa
from .comments import CommentsResource  # noqa
from .comment_threads import CommentThreadsResource  # noqa
from .i18n_languages import I18nLanguagesResource  # noqa
from .i18n_regions import I18nRegionsResource  # noqa
from .members import MembersResource  # noqa
from .membership_levels import MembershipLevelsResource  # noqa
from .playlist_items import PlaylistItemsResource  # noqa
from .playlists import PlaylistsResource  # noqa
from .search import SearchResource  # noqa
from .subscriptions import SubscriptionsResource  # noqa
from .thumbnails import ThumbnailsResource  # noqa
from .video_abuse_report_reasons import VideoAbuseReportReasonsResource  # noqa
from .video_categories import VideoCategoriesResource  # noqa
from .videos import VideosResource  # noqa
from .watermarks import WatermarksResource  # noqa



from dataclasses import dataclass, field
from typing import List, Optional

from .base import BaseModel


@dataclass
class AccessToken(BaseModel):
    """
    A class representing for access token.
    Refer: https://developers.google.com/youtube/v3/guides/auth/server-side-web-apps#exchange-authorization-code
    """

    access_token: Optional[str] = field(default=None)
    expires_in: Optional[int] = field(default=None)
    refresh_token: Optional[str] = field(default=None, repr=False)
    scope: Optional[List[str]] = field(default=None, repr=False)
    token_type: Optional[str] = field(default=None)
    expires_at: Optional[float] = field(default=None, repr=False)


@dataclass
class UserProfile(BaseModel):
    """
    A class representing for user profile.
    Refer: https://any-api.com/googleapis_com/oauth2/docs/userinfo/oauth2_userinfo_v2_me_get
    """

    id: Optional[str] = field(default=None)
    name: Optional[str] = field(default=None)
    given_name: Optional[str] = field(default=None, repr=False)
    family_name: Optional[str] = field(default=None, repr=False)
    picture: Optional[str] = field(default=None, repr=False)
    locale: Optional[str] = field(default=None, repr=False)
